'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _pixivAppApi = require('pixiv-app-api');

var _pixivAppApi2 = _interopRequireDefault(_pixivAppApi);

var _pixivImg = require('pixiv-img');

var _pixivImg2 = _interopRequireDefault(_pixivImg);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _yargs = require('yargs');

var _yargs2 = _interopRequireDefault(_yargs);

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _promiseSeries = require('promise-series');

var _promiseSeries2 = _interopRequireDefault(_promiseSeries);

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var PixivDownloaderCli = function () {
	function PixivDownloaderCli() {
		_classCallCheck(this, PixivDownloaderCli);

		this.state = {
			page: 0,
			output: false
		};

		this.spinner = (0, _ora2.default)({ color: 'red' });
		this.pixiv = new _pixivAppApi2.default();
		this.log = console.log;
		this.error = console.error;
	}

	_createClass(PixivDownloaderCli, [{
		key: 'entry',
		value: function entry(args) {
			if (typeof args === 'undefined') {
				return 1;
			}

			if ((typeof args === 'undefined' ? 'undefined' : _typeof(args)) === 'object') {
				args = args.join(' ');
			}

			if (typeof args !== 'string') {
				return 1;
			}

			args = (0, _yargs2.default)(args).describe('output', 'Output file').describe('version', 'Print version number and exit').help('help').alias('o', 'output').alias('h', 'help').alias('v', 'version').argv;

			if (args.version || args.v) {
				this.log(_package2.default.name + ' ' + _package2.default.version);
				return 0;
			}

			if ((args.output || args.o) && args._.length === 0) {
				this.error(_chalk2.default.bold.red('ERROR: Invalid arguments'));
				return 1;
			}

			if (args.output || args.o) {
				this.state.output = args.output || args.o;
			} else {
				this.state.output = 'pixiv_downloader_output';
			}

			if (args._.length > 0) {
				var tags = args._.join(' ');
				this.search(tags);
				return 0;
			}

			this.log('Usage: ' + _package2.default.name + ' <tag> -o <output>');
		}
	}, {
		key: 'search',
		value: function search(tags) {
			var _this = this;

			this.spinner.text = 'Getting search results from pixiv';
			this.pixiv.searchIllust(tags).then(function (json) {
				_this.handleSearchResults(json);
			});
		}
	}, {
		key: 'handleSearchResults',
		value: function handleSearchResults(json) {
			var _this2 = this;

			this.state.page++;
			this.spinner.start();
			this.spinner.text = 'Start Downloading';
			var series = new _promiseSeries2.default();
			json.illusts.forEach(function (illust) {
				if (illust.metaPages.length === 0) {
					series.add(function () {
						return _this2.download({
							id: illust.id,
							url: illust.imageUrls.large,
							title: illust.title,
							index: 0,
							length: 1
						});
					});
				} else {
					illust.metaPages.forEach(function (metaPage, index) {
						series.add(function () {
							return _this2.download({
								id: illust.id,
								url: metaPage.imageUrls.original,
								title: illust.title,
								index: index,
								length: illust.metaPages.length
							});
						});
					});
				}
			});
			series.run().then(function () {
				_this2.spinner.succeed('Page ' + _this2.state.page + ' downloaded successfully');
				if (_this2.pixiv.hasNext()) {
					_this2.spinner.text = 'Getting search results from pixiv';
					_this2.pixiv.next().then(function (json) {
						_this2.handleSearchResults(json);
					});
				} else {
					_this2.spinner.info('All pages have been downloaded');
				}
			});
		}
	}, {
		key: 'download',
		value: function download(_ref) {
			var id = _ref.id,
			    url = _ref.url,
			    title = _ref.title,
			    index = _ref.index,
			    length = _ref.length;

			var text = index + 1 + '/' + length + ' id' + id + ' ' + _chalk2.default.bold.underline.blue(title);
			this.spinner.text = text;
			var output = _path2.default.join(this.state.output, 'id' + id);
			_mkdirp2.default.sync(output);
			return (0, _pixivImg2.default)(url, _path2.default.join(output, _path2.default.basename(url)));
		}
	}]);

	return PixivDownloaderCli;
}();

exports.default = PixivDownloaderCli;